@@ -31,10 +31,12 @@
 #import <MediaPlayer/MediaPlayer.h>
 #import <HockeySDK/HockeySDK.h>
 #import "VLCSidebarController.h"
-#import "VLCKeychainCoordinator.h"
 #import "VLCActivityManager.h"
 #import "VLCDropboxConstants.h"
 #import <ObjectiveDropboxOfficial/ObjectiveDropboxOfficial.h>
+#import "VLCPlaybackNavigationController.h"
+#import "PAPasscodeViewController.h"
+#import "VLC_iOS-Swift.h"
 
 NSString *const VLCDropboxSessionWasAuthorized = @"VLCDropboxSessionWasAuthorized";
 
@@ -42,10 +44,10 @@ NSString *const VLCDropboxSessionWasAuthorized = @"VLCDropboxSessionWasAuthorize
 
 @interface VLCAppDelegate () <VLCMediaFileDiscovererDelegate>
 {
-    BOOL _passcodeValidated;
     BOOL _isRunningMigration;
     BOOL _isComingFromHandoff;
     VLCWatchCommunication *_watchCommunication;
+    VLCKeychainCoordinator *_keychainCoordinator;
 }
 
 @end
@@ -56,24 +58,18 @@ NSString *const VLCDropboxSessionWasAuthorized = @"VLCDropboxSessionWasAuthorize
 {
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
 
-    NSNumber *skipLoopFilterDefaultValue;
-    int deviceSpeedCategory = [[UIDevice currentDevice] VLCSpeedCategory];
-    if (deviceSpeedCategory < 3)
-        skipLoopFilterDefaultValue = kVLCSettingSkipLoopFilterNonKey;
-    else
-        skipLoopFilterDefaultValue = kVLCSettingSkipLoopFilterNonRef;
-
-    NSDictionary *appDefaults = @{kVLCSettingPasscodeAllowTouchID : @(1),
+    NSDictionary *appDefaults = @{kVLCSettingPasscodeAllowFaceID : @(1),
+                                  kVLCSettingPasscodeAllowTouchID : @(1),
                                   kVLCSettingContinueAudioInBackgroundKey : @(YES),
                                   kVLCSettingStretchAudio : @(NO),
                                   kVLCSettingTextEncoding : kVLCSettingTextEncodingDefaultValue,
-                                  kVLCSettingSkipLoopFilter : skipLoopFilterDefaultValue,
+                                  kVLCSettingSkipLoopFilter : kVLCSettingSkipLoopFilterNonRef,
                                   kVLCSettingSubtitlesFont : kVLCSettingSubtitlesFontDefaultValue,
                                   kVLCSettingSubtitlesFontColor : kVLCSettingSubtitlesFontColorDefaultValue,
                                   kVLCSettingSubtitlesFontSize : kVLCSettingSubtitlesFontSizeDefaultValue,
                                   kVLCSettingSubtitlesBoldFont: kVLCSettingSubtitlesBoldFontDefaultValue,
                                   kVLCSettingDeinterlace : kVLCSettingDeinterlaceDefaultValue,
-                                  kVLCSettingHWDecoding : kVLCSettingHWDecodingDefault,
+                                  kVLCSettingHardwareDecoding : kVLCSettingHardwareDecodingDefault,
                                   kVLCSettingNetworkCaching : kVLCSettingNetworkCachingDefaultValue,
                                   kVLCSettingVolumeGesture : @(YES),
                                   kVLCSettingPlayPauseGesture : @(YES),
@@ -105,22 +101,7 @@ NSString *const VLCDropboxSessionWasAuthorized = @"VLCDropboxSessionWasAuthorize
     // Configure Dropbox
     [DBClientsManager setupWithAppKey:kVLCDropboxAppKey];
 
-    /* listen to validation notification */
-    [[NSNotificationCenter defaultCenter] addObserver:self
-                                             selector:@selector(passcodeWasValidated:)
-                                                 name:VLCPasscodeValidated
-                                               object:nil];
-
-    // Change the keyboard for UISearchBar
-    [[UITextField appearance] setKeyboardAppearance:UIKeyboardAppearanceDark];
-    // For the cursor
-    [[UITextField appearance] setTintColor:[UIColor VLCOrangeTintColor]];
-    // Don't override the 'Cancel' button color in the search bar with the previous UITextField call. Use the default blue color
-    [[UIBarButtonItem appearanceWhenContainedIn:[UISearchBar class], nil] setTitleTextAttributes:@{NSForegroundColorAttributeName: [UIColor colorWithRed:0.0 green:122.0/255.0 blue:1.0 alpha:1.0]} forState:UIControlStateNormal];
-    // For the edit selection indicators
-    [[UITableView appearance] setTintColor:[UIColor VLCOrangeTintColor]];
-
-    [[UISwitch appearance] setOnTintColor:[UIColor VLCOrangeTintColor]];
+    [self setupAppearence];
 
     // Init the HTTP Server and clean its cache
     [[VLCHTTPUploaderController sharedInstance] cleanCache];
@@ -128,20 +109,25 @@ NSString *const VLCDropboxSessionWasAuthorized = @"VLCDropboxSessionWasAuthorize
     self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
     // enable crash preventer
     void (^setupBlock)() = ^{
-        _libraryViewController = [[VLCLibraryViewController alloc] init];
-        VLCSidebarController *sidebarVC = [VLCSidebarController sharedInstance];
-        VLCNavigationController *navCon = [[VLCNavigationController alloc] initWithRootViewController:_libraryViewController];
-        sidebarVC.contentViewController = navCon;
+        __weak typeof(self) weakSelf = self;
+        void (^setupLibraryBlock)() = ^{
+            _libraryViewController = [[VLCLibraryViewController alloc] init];
+            UINavigationController *navCon = [[UINavigationController alloc] initWithRootViewController:_libraryViewController];
 
-        VLCPlayerDisplayController *playerDisplayController = [VLCPlayerDisplayController sharedInstance];
-        playerDisplayController.childViewController = sidebarVC.fullViewController;
+            VLCSidebarController *sidebarVC = [VLCSidebarController sharedInstance];
+            sidebarVC.contentViewController = navCon;
 
-        self.window.rootViewController = playerDisplayController;
+            VLCPlayerDisplayController *playerDisplayController = [VLCPlayerDisplayController sharedInstance];
+            playerDisplayController.childViewController = sidebarVC.fullViewController;
+
+            weakSelf.window.rootViewController = playerDisplayController;
+        };
+        UINavigationController *navCon = [[UINavigationController alloc] initWithRootViewController:[[UIViewController alloc] init]];
+        self.window.rootViewController = navCon;
         [self.window makeKeyAndVisible];
+        [self validatePasscodeIfNeededWithCompletion:setupLibraryBlock];
 
-        [self validatePasscode];
-
-        BOOL spotlightEnabled = ![[VLCKeychainCoordinator defaultCoordinator] passcodeLockEnabled];
+        BOOL spotlightEnabled = ![VLCKeychainCoordinator passcodeLockEnabled];
         [[MLMediaLibrary sharedMediaLibrary] setSpotlightIndexingEnabled:spotlightEnabled];
         [[MLMediaLibrary sharedMediaLibrary] applicationWillStart];
 
@@ -216,6 +202,27 @@ NSString *const VLCDropboxSessionWasAuthorized = @"VLCDropboxSessionWasAuthorize
     return YES;
 }
 
+- (void)setupAppearence
+{
+    UIColor *vlcOrange = [UIColor VLCOrangeTintColor];
+    // Change the keyboard for UISearchBar
+    [[UITextField appearance] setKeyboardAppearance:UIKeyboardAppearanceDark];
+    // For the cursor
+    [[UITextField appearance] setTintColor:vlcOrange];
+    // Don't override the 'Cancel' button color in the search bar with the previous UITextField call. Use the default blue color
+    [[UIBarButtonItem appearanceWhenContainedInInstancesOfClasses:@[[UISearchBar class]]] setTitleTextAttributes:@{[UIColor whiteColor] : NSForegroundColorAttributeName} forState:UIControlStateNormal];
+
+    [[UINavigationBar appearance] setBarTintColor:vlcOrange];
+    [[UINavigationBar appearanceWhenContainedInInstancesOfClasses:@[[VLCPlaybackNavigationController class]]] setBarTintColor: nil];
+    [[UINavigationBar appearance] setTintColor:[UIColor whiteColor]];
+    [[UINavigationBar appearance] setTitleTextAttributes: @{ NSForegroundColorAttributeName : [UIColor whiteColor] }];
+    // For the edit selection indicators
+    [[UITableView appearance] setTintColor:vlcOrange];
+    [[UISwitch appearance] setOnTintColor:vlcOrange];
+    [[UISearchBar appearance] setBarTintColor:vlcOrange];
+    [[UISearchBar appearance] setTintColor:[UIColor whiteColor]];
+}
+
 - (void)dealloc
 {
     [[NSNotificationCenter defaultCenter] removeObserver:self];
@@ -285,10 +292,7 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
     }
 }
 
-- (BOOL)application:(UIApplication *)application
-            openURL:(NSURL *)url
-  sourceApplication:(NSString *)sourceApplication
-         annotation:(id)annotation
+- (BOOL)application:(UIApplication *)app openURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey,id> *)options
 {
     //Handles Dropbox Authorization flow.
     DBOAuthResult *authResult = [DBClientsManager handleRedirectURL:url];
@@ -305,18 +309,25 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
     }
 
     if (_libraryViewController && url != nil) {
-        APLog(@"%@ requested %@ to be opened", sourceApplication, url);
+        APLog(@"requested %@ to be opened", url);
 
         if (url.isFileURL) {
             NSArray *searchPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
-            NSString *directoryPath = searchPaths[0];
+            NSString *directoryPath = searchPaths.firstObject;
             NSURL *destinationURL = [NSURL fileURLWithPath:[NSString stringWithFormat:@"%@/%@", directoryPath, url.lastPathComponent]];
             NSError *theError;
+            NSFileManager *manager = [NSFileManager defaultManager];
             [[NSFileManager defaultManager] moveItemAtURL:url toURL:destinationURL error:&theError];
             if (theError.code != noErr)
                 APLog(@"saving the file failed (%li): %@", (long)theError.code, theError.localizedDescription);
 
             [[VLCMediaFileDiscoverer sharedInstance] updateMediaList];
+
+            NSURLRelationship relationship;
+            [manager getRelationship:&relationship ofDirectoryAtURL:[NSURL fileURLWithPath:directoryPath] toItemAtURL:url error:&theError];
+            if (relationship == NSURLRelationshipContains) {
+                [self playWithURL:url successCallback:nil errorCallback:nil];
+            }
         } else if ([url.scheme isEqualToString:@"vlc-x-callback"] || [url.host isEqualToString:@"x-callback-url"]) {
             // URL confirmes to the x-callback-url specification
             // vlc-x-callback://x-callback-url/action?param=value&x-success=callback
@@ -342,9 +353,7 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
                     errorCallback = [NSURL URLWithString:value];
             }
             if ([action isEqualToString:@"stream"] && movieURL) {
-                VLCPlaybackController *vpc = [VLCPlaybackController sharedInstance];
-                vpc.fullscreenSessionRequested = YES;
-                [vpc playURL:movieURL successCallback:successCallback errorCallback:errorCallback];
+                [self playWithURL:movieURL successCallback:successCallback errorCallback:errorCallback];
             }
             else if ([action isEqualToString:@"download"] && movieURL) {
                 [self downloadMovieFromURL:movieURL fileNameOfMedia:fileName];
@@ -379,15 +388,12 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
                     if (cancelled)
                         [self downloadMovieFromURL:url fileNameOfMedia:nil];
                     else {
-                        VLCPlaybackController *vpc = [VLCPlaybackController sharedInstance];
-                        [vpc playURL:url successCallback:nil errorCallback:nil];
+                        [self playWithURL:url successCallback:nil errorCallback:nil];
                     }
                 };
                 [alert show];
             } else {
-                VLCPlaybackController *vpc = [VLCPlaybackController sharedInstance];
-                vpc.fullscreenSessionRequested = YES;
-                [vpc playURL:url successCallback:nil errorCallback:nil];
+                [self playWithURL:url successCallback:nil errorCallback:nil];
             }
         }
         return YES;
@@ -402,9 +408,20 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
 
 - (void)applicationWillResignActive:(UIApplication *)application
 {
-    _passcodeValidated = NO;
-    [self.libraryViewController setEditing:NO animated:NO];
-    [self validatePasscode];
+    //Touch ID is shown 
+    if ([_window.rootViewController.presentedViewController isKindOfClass:[UINavigationController class]]){
+        UINavigationController *navCon = (UINavigationController *)_window.rootViewController.presentedViewController;
+        if ([navCon.topViewController isKindOfClass:[PAPasscodeViewController class]]){
+            return;
+        }
+    }
+    __weak typeof(self) weakself = self;
+    [self validatePasscodeIfNeededWithCompletion:^{
+        [weakself.libraryViewController updateViewContents];
+        if ([VLCPlaybackController sharedInstance].isPlaying){
+            [[VLCPlayerDisplayController sharedInstance] pushPlaybackView];
+        }
+    }];
     [[MLMediaLibrary sharedMediaLibrary] applicationWillExit];
 }
 
@@ -418,12 +435,6 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
     }
 }
 
-- (void)applicationWillTerminate:(UIApplication *)application
-{
-    _passcodeValidated = NO;
-    [[NSUserDefaults standardUserDefaults] synchronize];
-}
-
 - (void)application:(UIApplication *)application performActionForShortcutItem:(UIApplicationShortcutItem *)shortcutItem completionHandler:(void (^)(BOOL))completionHandler
 {
     [[VLCSidebarController sharedInstance] performActionForShortcutItem:shortcutItem];
@@ -460,31 +471,21 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
 }
 
 #pragma mark - pass code validation
-
-- (void)passcodeWasValidated:(NSNotification *)aNotifcation
+- (VLCKeychainCoordinator *)keychainCoordinator
 {
-    _passcodeValidated = YES;
-    [self.libraryViewController updateViewContents];
-    if ([VLCPlaybackController sharedInstance].isPlaying)
-        [[VLCPlayerDisplayController sharedInstance] pushPlaybackView];
+    if (!_keychainCoordinator) {
+        _keychainCoordinator = [[VLCKeychainCoordinator alloc] init];
+    }
+    return _keychainCoordinator;
 }
 
-- (BOOL)passcodeValidated
+- (void)validatePasscodeIfNeededWithCompletion:(void(^)(void))completion
 {
-    return _passcodeValidated;
-}
-
-- (void)validatePasscode
-{
-    VLCKeychainCoordinator *keychainCoordinator = [VLCKeychainCoordinator defaultCoordinator];
-
-    if (!_passcodeValidated && [keychainCoordinator passcodeLockEnabled]) {
+    if ([VLCKeychainCoordinator passcodeLockEnabled]) {
         [[VLCPlayerDisplayController sharedInstance] dismissPlaybackView];
-
-        [keychainCoordinator validatePasscode];
+        [self.keychainCoordinator validatePasscodeWithCompletion:completion];
     } else {
-        _passcodeValidated = YES;
-        [self passcodeValidated];
+        completion();
     }
 }
 
@@ -498,10 +499,15 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
                                                  scrollPosition:UITableViewScrollPositionNone];
 }
 
-#pragma mark - remote events pre 7.1
-
-- (void)remoteControlReceivedWithEvent:(UIEvent *)event {
-    [[VLCPlaybackController sharedInstance] remoteControlReceivedWithEvent:event];
+#pragma mark - playback
+- (void)playWithURL:(NSURL *)url successCallback:(NSURL *)successCallback errorCallback:(NSURL *)errorCallback
+{
+    VLCPlaybackController *vpc = [VLCPlaybackController sharedInstance];
+    vpc.fullscreenSessionRequested = YES;
+    vpc.successCallback = successCallback;
+    vpc.errorCallback = errorCallback;
+    VLCMediaList *mediaList = [[VLCMediaList alloc] initWithArray:@[[VLCMedia mediaWithURL:url]]];
+    [vpc playMediaList:mediaList firstIndex:0 subtitlesFilePath:nil];
 }
 
 #pragma mark - watch stuff
@@ -514,6 +520,4 @@ handleWatchKitExtensionRequest:(NSDictionary *)userInfo
     }
 }
 
-
-
 @end
